<?php

namespace Drupal\elasticsearch_connector_suggester\EventSubscriber;

use Drupal\Component\EventDispatcher\Event;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\elasticsearch_connector\Event\QueryParamsEvent;
use Drupal\elasticsearch_connector_suggester\Event\FieldNameEvent;
use Drupal\elasticsearch_connector_suggester\Event\KeysEvent;
use Drupal\elasticsearch_connector_suggester\Event\RouteNameEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * This is the QueryParamsEventSubscriber class.
 */
final class QueryParamsEventSubscriber implements EventSubscriberInterface {

  use SubscriberTrait;

  /**
   * Constructs a QueryParamsEventSubscriber object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly EventDispatcherInterface $eventDispatcher,
    private readonly RouteMatchInterface $routeMatch,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[QueryParamsEvent::class] = [
      'onElasticsearchBuildQuery',
    ];

    return $events;
  }

  /**
   * Called on elasticsearch_connector QueryParamsEvent.
   *
   * Replace query to suggester request.
   *
   * @param \Drupal\elasticsearch_connector\Event\QueryParamsEvent $event
   *   The event.
   */
  public function onElasticsearchBuildQuery(Event $event) {
    // The autocomplete route name.
    $route_name = 'search_api_autocomplete.autocomplete';
    /** @var \Drupal\elasticsearch_connector_suggester\Event\RouteNameEvent $name_event */
    $name_event = new RouteNameEvent($route_name);
    // Allow modification route name via an event.
    $this->eventDispatcher->dispatch($name_event);
    $route_name = $name_event->getName();
    if ($this->routeMatch->getRouteName() !== $route_name) {
      return;
    }

    $params = $event->getParams();
    $index_name = $params['index'] ?? NULL;
    if ($index_name === NULL) {
      return;
    }
    $index = $this->loadIndexFromIndexName($index_name);
    $fields = $this->getFields($index);
    if (empty($fields)) {
      return;
    }

    $delta = 0;
    $field = NULL;
    $field_name = $params['body']['query']['query_string']['fields'][$delta] ?? NULL;
    if ($field_name === NULL) {
      // Field name not found.
      return;
    }
    $autocomplete_name = $this->getAutocompleteField($fields, $field, $field_name);
    if ($field === NULL) {
      // Field not found.
      return;
    }
    /** @var \Drupal\elasticsearch_connector_suggester\Event\FieldNameEvent $field_name_event */
    $field_name_event = new FieldNameEvent($field_name, $autocomplete_name, $params);
    // Allow modification field name via an event.
    $this->eventDispatcher->dispatch($field_name_event);
    $autocomplete_name = $field_name_event->getAutocompleteName();
    if ($autocomplete_name === NULL) {
      // Query field name not contains autocomplete field name.
      return;
    }

    // Get keys from query.
    $keys = $params['body']['query']['query_string']['query'] ?? '';
    // Remove not needed words and characters.
    $prefix = preg_replace('/( AND|~)/', '', $keys);
    $keys_event = new KeysEvent($keys, $params);
    // Allow modification keys via an event.
    $this->eventDispatcher->dispatch($keys_event);
    $prefix = $keys_event->getKeys();

    // Set field name to completion.
    $field .= '.completion';

    // @todo maybe needs to configurable in the future.
    // Set size to 10 for autocomplete.
    $size = 10;

    // Set suggest request.
    $params['body'] = [
      'suggest' => [
        'autocomplete' => [
          'prefix' => $prefix,
          'completion' => [
            'field' => $field,
            'skip_duplicates' => TRUE,
            'size' => $size,
          ],
        ],
      ],
    ];

    $event->setParams($params);
  }

  /**
   * Get autocomplete field.
   *
   * @param array $fields
   *   The fields.
   * @param string|null $field
   *   The field.
   * @param string $field_name
   *   The field name.
   *
   * @return string|null
   *   The autocomplete field.
   */
  protected function getAutocompleteField(array $fields, &$field, string $field_name):? string {
    $autocomplete_field = NULL;
    foreach ($fields as $field) {
      if (strpos($field_name, $field) !== FALSE) {
        $autocomplete_field = $field;
        break;
      }
    }

    return $autocomplete_field;
  }

}
