<?php

namespace Drupal\elasticsearch_connector_suggester\EventSubscriber;

use Drupal\Core\Entity\EntityInterface;
use Drupal\search_api\IndexInterface;

/**
 * Trait for event subscribers.
 */
trait SubscriberTrait {

  /**
   * Get autocomplete fields.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The index name.
   *
   * @return array
   *   The fields.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getFields(IndexInterface $index): array {
    $fields = [];

    // Load all autocomplete entities.
    $autocompletes = $this->entityTypeManager
      ->getStorage('search_api_autocomplete_search')
      ->loadMultiple();

    /** @var \Drupal\search_api_autocomplete\Entity\Search $value */
    foreach ($autocompletes as $value) {
      // Get autocomplete settings for the index.
      if ($value->getIndexId() == $index->id()) {
        /** @var \Drupal\search_api_autocomplete\Entity\Search $autocomplete */
        $autocomplete = $value;
        break;
      }
    }

    if (!isset($autocomplete)) {
      // No autocomplete settings for the index.
      return $fields;
    }

    /** @var \Drupal\search_api_autocomplete\Suggester\SuggesterInterface $suggester */
    foreach ($autocomplete->getSuggesters() as $suggester) {
      if ($suggester->getPluginId() != 'elastic_live_results') {
        // Not elasticsearch suggester.
        continue;
      }
      // Get fields for the suggester.
      $fields[] = $suggester->getConfiguration()['fields'] ?? [];
    }

    return array_merge(...$fields);
  }

  /**
   * Loads the index entity associated with this event.
   *
   * @param string $index_name
   *   The long index name as a string.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The loaded index or NULL.
   */
  protected function loadIndexFromIndexName($index_name):? EntityInterface {
    /** @var \Drupal\search_api\Entity\Index[] $search_api_indexes */
    $search_api_indexes = $this->entityTypeManager
      ->getStorage('search_api_index')
      ->loadMultiple();

    /** @var \Drupal\search_api\Entity\Index $search_api_index */
    foreach ($search_api_indexes as $search_api_index) {
      if ($index_name === $this->getIndexName($search_api_index)) {
        return $search_api_index;
      }
    }

    return NULL;
  }

  /**
   * Return the index name.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The index.
   *
   * @return string
   *   The index name.
   */
  protected function getIndexName(IndexInterface $index): string {
    // Elasticsearch BackendClientInterface::getIndexId() method is not defined.
    // @phpstan-ignore-next-line
    return $index->getServerInstance()
      ->getBackend()
      ->getBackendClient()
      ->getIndexId($index);
  }

}
